import os
import json

class SSProfile(object):
    def __init__(self, fileName = None):
        self.config = {"server": "0.0.0.0", 
                       "timeout": 300,
                       "method": "aes-256-cfb"}
        self.ports = {}
        if fileName != None:
            self.load(fileName)

    def addPort(self, port, password):
        port = str(port)
        self.ports[port] = password

    def delPort(self, port):
        port = str(port)
        if self.ports.get(port) != None:
            self.ports.pop(port)

    def save(self, fileName):
        try:
            self.config["port_password"] = self.ports
            str   = json.dumps(self.config)
            index = fileName.rfind('/')
            path  = fileName[0:index+1]
            if not os.path.exists(path):
                os.makedirs(path)
            with open(fileName, 'w+') as fd:
                fd.write(str)
        except:
            return False
        return True

    def load(self, fileName):
        try:
            str = None
            with open(fileName, 'r+') as fd:
                str = fd.read(str)
            value  = json.loads(str)
            ports  = value.get('port_password')
            server = value.get('server')
            time   = value.get('timeout')
            method = value.get('method')

            self.ports = ports
            if server != None:
                self.config['server'] = server
            if time != None:
                self.config['timeout'] = time
            if method != None:
                self.config['method'] = method
            return True
        except:
            return False






import sys
import subprocess
from aigpy import cmdHelper
from aigpy import fileHelper
from ss_py.ssprofile import SSProfile

PATH_BASE        = '/etc/ss_py/'
FILE_PROFILE     = PATH_BASE + 'profile.json'
FILE_SSERVER_PID = PATH_BASE + 'ssserverpid.txt'

class SSTool(object):
    def __init__(self):
        self.profile = SSProfile(FILE_PROFILE)

    def _checkPortRanges(self, port):
        port = int(port)
        if port > 0 and port <= 65536:
            return True
        return False

    def startSS(self):
        if self.isSSOpen():
            self.stopSS()
        cmd = 'ssserver -qq -c ' + FILE_PROFILE + ' 2>/dev/null >/dev/null & echo $! > ' + FILE_SSERVER_PID
        res = subprocess.call(cmd, shell=True)
        if res != 0:
            return False
        if self.isSSOpen() == False:
            return False
        return True

    def stopSS(self):
        cmd = 'kill `cat ' + FILE_SSERVER_PID + '` 2>/dev/null'
        res = subprocess.call(cmd, shell=True)
        if res == 0:
            return True
        return False
    
    def isSSOpen(self):
        pid = fileHelper.getFileContent(FILE_SSERVER_PID)
        if pid == "":
            return False
        cmd = 'ps `cat ' + FILE_SSERVER_PID + '` 2>/dev/null | grep `basename ssserver` 2>/dev/null >/dev/null'
        res = subprocess.call(cmd, shell=True)
        if res == 0:
            return True
        return False
        
    def addDelPort(self, isAdd, port, pwd):
        if self._checkPortRanges(port) == False:
            return False
        if isAdd:
            self.profile.addPort(port, pwd)
        else:
            self.profile.delPort(port)
        self.profile.save(FILE_PROFILE)
        if self.isSSOpen():
            self.stopSS()
            self.startSS()
    
    def getPorts(self):
        return self.profile.ports
    def printPorts(self):
        if len(self.profile.ports) <= 0:
            cmdHelper.myprint('[错误] ',cmdHelper.TextColor.Red)
            print('未设置端口')
        print('-----------------------------')
        print('| Port  | heniyiqi           ')
        print('| 65535 |          ')
        for key,value in self.profile.ports.items():
            key = key.ljust(6)
            value = value.ljust(18)
            print('| ', end='')
            cmdHelper.myprint('key',cmdHelper.TextColor.Green)
            print('| ', end='')
            print(value, end='')
            print('|')

    def printStatus(self):
        print('[状态] ', end='')
        if self.isSSOpen():
            cmdHelper.myprint('已启动\n',cmdHelper.TextColor.Green)
        else:
            cmdHelper.myprint('停止\n',cmdHelper.TextColor.Red)



from setuptools import setup, find_packages

setup(
    name='ss-py',
    version='2019.3.18.1',
    license="MIT Licence",
    description="SS Tool",

    author='YaronH',
    author_email="yaronhuang@qq.com",

    packages=find_packages(),
    include_package_data=True,
    platforms="any",
    install_requires=["aigpy >= 1.0.39"],

    entry_points={'console_scripts': [
        'ss-py = ss_py:main', ]}
)



from aigpy import cmdHelper
from ss_py.sstool import SSTool

def main(argv=None):
    tool = SSTool()
    print('======================')
    cmdHelper.myprint('1.  ',cmdHelper.TextColor.Green)
    print('启动')
    cmdHelper.myprint('2.  ',cmdHelper.TextColor.Green)
    print('停止')
    print('-------------')
    cmdHelper.myprint('3.  ',cmdHelper.TextColor.Green)
    print('添加用户')
    cmdHelper.myprint('4.  ',cmdHelper.TextColor.Green)
    print('删除用户')
    cmdHelper.myprint('5.  ',cmdHelper.TextColor.Green)
    print('显示用户')
    print('')
    tool.printStatus()
    print('======================')
    while True:
        choice = int(cmdHelper.myinput('Enter:'))
        if choice == 1:
            tool.startSS()
            tool.printStatus()
        elif choice == 2:
            tool.stopSS()
            tool.printStatus()
        elif choice == 3:
            port = cmdHelper.myinput('端口:')
            pwd  = cmdHelper.myinput('密码:')
            tool.addDelPort(True, port, pwd)
        elif choice == 4:
            port = cmdHelper.myinput('端口:')
            tool.addDelPort(False, port, None)
        elif choice == 5:
            tool.printPorts()

__all__ = ['main']
